# -*- coding: utf-8 -*-
"""
Created on Sun Dec  8 17:15:22 2024

@author: mbmad
"""
from autosleap.files import job_batchfile_create
from datetime import datetime
import subprocess
import threading

def now_str():
    return datetime.now().strftime('%Y-%m-%d-%H-%M-%S')

class JobInterface():
    def __init__(self, sourcefile, destfile, token, settings, reporter):
        self.sourcefile = sourcefile
        self.destfile = destfile
        self.token = token
        self.settings = settings
        self.reporter = reporter
        
        self.batch_fpath = job_batchfile_create(
            '_'.join([now_str(),self.job_type()[1], token,'.bat']))
    
    def run(self):
        """
        create batch file then run it using the run_batch function
        """
        if ' ' in self.token:
            self.reporter.job_print('WARNING!!!! SPACES DETECTED IN FILENAME. CHANGE FILENAME OR JOB WILL FAIL!!!!')
            return False
        with open(self.batch_fpath,'w') as file:
            file.write(self.job_construct_batch_contents())
        run_batch(self.batch_fpath, self.reporter)
        return True
            
    def job_construct_batch_contents(self):
        raise NotImplementedError('method job_batch_contents not implemented')
    
    def job_type(self):
        raise NotImplementedError('method job_type not implemented')

def read_output_subprocess(pipe, report_function):
    try:
        for line in iter(pipe.readline, ''):
            report_function(line, end='', flush=True)
    finally:
        pipe.close()

def run_batch(bat_file_path, reporter):
    try:
        # Create subprocess to execute batch file generated by job
        process = subprocess.Popen(
            ['cmd.exe', '/c', bat_file_path],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            bufsize=1,
            universal_newlines=True
        )

        # Create threads to read stdout and stderr
        stdout_thread = threading.Thread(target=read_output_subprocess, args=(process.stdout, reporter.job_print))
        stderr_thread = threading.Thread(target=read_output_subprocess, args=(process.stderr, reporter.job_print))

        # Start the threads
        stdout_thread.start()
        stderr_thread.start()

        # Wait for the threads to finish
        stdout_thread.join()
        stderr_thread.join()
        
        # Wait for the subprocess to finish
        process.wait()

        if process.returncode == 0:
            reporter.job_print("\nBatch file executed successfully.")
        else:
            reporter.job_print("\nError occurred while executing the batch file.")
            
    except subprocess.CalledProcessError as e:
        reporter.job_print(f"Error occurred: {e}")